import os

NAME = "abcli"

ICON = "🪄"

DESCRIPTION = f"{ICON} a language to speak AI."

VERSION = "9.310.1"

REPO_NAME = "awesome-bash-cli"

MARQUEE = "https://github.com/kamangir/awesome-bash-cli/raw/main/assets/marquee.png"
